@@ -1,15 +1,10 @@
 //! Tests for the `cargo bench` command.
 
-use cargo_test_support::is_nightly;
 use cargo_test_support::paths::CargoPathExt;
 use cargo_test_support::{basic_bin_manifest, basic_lib_manifest, basic_manifest, project};
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn cargo_bench_simple() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file(
@@ -51,12 +46,8 @@ fn cargo_bench_simple() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_bench_implicit() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "src/main.rs",
@@ -99,12 +90,8 @@ fn bench_bench_implicit() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_bin_implicit() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "src/main.rs",
@@ -146,12 +133,8 @@ fn bench_bin_implicit() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_tarname() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "benches/bin1.rs",
@@ -183,12 +166,8 @@ fn bench_tarname() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_multiple_targets() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "benches/bin1.rs",
@@ -223,12 +202,8 @@ fn bench_multiple_targets() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn cargo_bench_verbose() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file(
@@ -255,12 +230,8 @@ fn cargo_bench_verbose() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn many_similar_names() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "src/lib.rs",
@@ -302,12 +273,8 @@ fn many_similar_names() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn cargo_bench_failing_test() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file(
@@ -346,27 +313,22 @@ fn cargo_bench_failing_test() {
 [FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
         )
-        .with_either_contains(
-            "[..]thread '[..]' panicked at 'assertion failed: `(left == right)`[..]",
-        )
-        .with_either_contains("[..]left: `\"hello\"`[..]")
-        .with_either_contains("[..]right: `\"nope\"`[..]")
-        .with_either_contains("[..]src/main.rs:15[..]")
+        .with_stdout_contains("[..]thread '[..]' panicked at[..]")
+        .with_stdout_contains("[..]assertion failed[..]")
+        .with_stdout_contains("[..]left: `\"hello\"`[..]")
+        .with_stdout_contains("[..]right: `\"nope\"`[..]")
+        .with_stdout_contains("[..]src/main.rs:15[..]")
         .with_status(101)
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_with_lib_dep() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
                 authors = []
@@ -424,12 +386,8 @@ fn bench_with_lib_dep() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_with_deep_lib_dep() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .at("bar")
         .file(
@@ -487,17 +445,13 @@ fn bench_with_deep_lib_dep() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn external_bench_explicit() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
                 authors = []
@@ -546,12 +500,8 @@ fn external_bench_explicit() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn external_bench_implicit() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "src/lib.rs",
@@ -593,17 +543,13 @@ fn external_bench_implicit() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_autodiscover_2015() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
                 authors = []
@@ -670,12 +616,8 @@ https://github.com/rust-lang/cargo/issues/5330
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn dont_run_examples() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("src/lib.rs", "")
         .file(
@@ -686,12 +628,8 @@ fn dont_run_examples() {
     p.cargo("bench").run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn pass_through_command_line() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "src/lib.rs",
@@ -727,12 +665,8 @@ fn pass_through_command_line() {
 
 // Regression test for running cargo-bench twice with
 // tests in an rlib
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn cargo_bench_twice() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("Cargo.toml", &basic_lib_manifest("foo"))
         .file(
@@ -754,17 +688,13 @@ fn cargo_bench_twice() {
     }
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn lib_bin_same_name() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
                 authors = []
@@ -811,12 +741,8 @@ fn lib_bin_same_name() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn lib_with_standard_name() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("Cargo.toml", &basic_manifest("syntax", "0.0.1"))
         .file(
@@ -861,12 +787,8 @@ fn lib_with_standard_name() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn lib_with_standard_name2() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -911,12 +833,8 @@ fn lib_with_standard_name2() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_dylib() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -1005,12 +923,8 @@ fn bench_dylib() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_twice_with_build_cmd() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -1054,12 +968,8 @@ fn bench_twice_with_build_cmd() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_with_examples() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -1141,17 +1051,13 @@ fn bench_with_examples() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn test_a_bench() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 authors = []
                 version = "0.1.0"
@@ -1181,12 +1087,8 @@ fn test_a_bench() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn test_bench_no_run() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file("src/lib.rs", "")
         .file(
@@ -1209,21 +1111,48 @@ fn test_bench_no_run() {
             "\
 [COMPILING] foo v0.0.1 ([..])
 [FINISHED] bench [optimized] target(s) in [..]
+[EXECUTABLE] benches src/lib.rs (target/release/deps/foo-[..][EXE])
+[EXECUTABLE] benches/bbaz.rs (target/release/deps/bbaz-[..][EXE])
 ",
         )
         .run();
 }
 
-#[cargo_test]
-fn test_bench_no_fail_fast() {
-    if !is_nightly() {
-        return;
-    }
+#[cargo_test(nightly, reason = "bench")]
+fn test_bench_no_run_emit_json() {
+    let p = project()
+        .file("src/lib.rs", "")
+        .file(
+            "benches/bbaz.rs",
+            r#"
+                #![feature(test)]
 
+                extern crate test;
+
+                use test::Bencher;
+
+                #[bench]
+                fn bench_baz(_: &mut Bencher) {}
+            "#,
+        )
+        .build();
+
+    p.cargo("bench --no-run --message-format json")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.0.1 ([..])
+[FINISHED] bench [optimized] target(s) in [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test(nightly, reason = "bench")]
+fn test_bench_no_fail_fast() {
     let p = project()
         .file("Cargo.toml", &basic_bin_manifest("foo"))
         .file(
-            "src/foo.rs",
+            "src/main.rs",
             r#"
             #![feature(test)]
             #[cfg(test)]
@@ -1247,29 +1176,46 @@ fn test_bench_no_fail_fast() {
             }
             "#,
         )
+        .file(
+            "benches/b1.rs",
+            r#"
+                #![feature(test)]
+                extern crate test;
+                #[bench]
+                fn b1_fail(_b: &mut test::Bencher) { assert_eq!(1, 2); }
+            "#,
+        )
         .build();
 
     p.cargo("bench --no-fail-fast -- --test-threads=1")
         .with_status(101)
-        .with_stderr_contains("[RUNNING] [..] (target/release/deps/foo-[..][EXE])")
+        .with_stderr(
+            "\
+[COMPILING] foo v0.5.0 [..]
+[FINISHED] bench [..]
+[RUNNING] unittests src/main.rs (target/release/deps/foo[..])
+[ERROR] bench failed, to rerun pass `--bin foo`
+[RUNNING] benches/b1.rs (target/release/deps/b1[..])
+[ERROR] bench failed, to rerun pass `--bench b1`
+[ERROR] 2 targets failed:
+    `--bin foo`
+    `--bench b1`
+",
+        )
         .with_stdout_contains("running 2 tests")
-        .with_stderr_contains("[RUNNING] [..] (target/release/deps/foo-[..][EXE])")
         .with_stdout_contains("test bench_hello [..]")
         .with_stdout_contains("test bench_nope [..]")
+        .with_stdout_contains("test b1_fail [..]")
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn test_bench_multiple_packages() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 authors = []
                 version = "0.1.0"
@@ -1289,7 +1235,7 @@ fn test_bench_multiple_packages() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "bar"
                 authors = []
                 version = "0.1.0"
@@ -1319,7 +1265,7 @@ fn test_bench_multiple_packages() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "baz"
                 authors = []
                 version = "0.1.0"
@@ -1352,17 +1298,13 @@ fn test_bench_multiple_packages() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_all_workspace() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.1.0"
 
@@ -1409,17 +1351,13 @@ fn bench_all_workspace() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_all_exclude() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.1.0"
 
@@ -1458,17 +1396,13 @@ test bar ... bench:           [..] ns/iter (+/- [..])",
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_all_exclude_glob() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.1.0"
 
@@ -1507,12 +1441,8 @@ test bar ... bench:           [..] ns/iter (+/- [..])",
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_all_virtual_manifest() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -1560,12 +1490,8 @@ fn bench_all_virtual_manifest() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_virtual_manifest_glob() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -1614,17 +1540,13 @@ fn bench_virtual_manifest_glob() {
 }
 
 // https://github.com/rust-lang/cargo/issues/4287
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn legacy_bench_name() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.1.0"
 
@@ -1656,12 +1578,8 @@ please set bench.path in Cargo.toml",
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn bench_virtual_manifest_all_implied() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "Cargo.toml",
@@ -1706,12 +1624,8 @@ fn bench_virtual_manifest_all_implied() {
         .run();
 }
 
-#[cargo_test]
+#[cargo_test(nightly, reason = "bench")]
 fn json_artifact_includes_executable_for_benchmark() {
-    if !is_nightly() {
-        return;
-    }
-
     let p = project()
         .file(
             "benches/benchmark.rs",
