@@ -18,7 +18,7 @@ fn feature_required() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
@@ -42,7 +42,7 @@ Caused by:
         .run();
 
     // Same, but stable.
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -97,7 +97,7 @@ fn feature_required_dependency() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
@@ -126,7 +126,7 @@ Caused by:
         .run();
 
     // Same, but stable.
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -173,7 +173,7 @@ fn unknown_feature() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -202,13 +202,13 @@ fn stable_feature_warns() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .with_stderr(
             "\
 warning: the cargo feature `test-dummy-stable` has been stabilized in the 1.0 \
 release and is no longer necessary to be listed in the manifest
   See https://doc.rust-lang.org/[..]cargo/ for more information about using this feature.
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
@@ -233,17 +233,17 @@ fn allow_features() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("-Zallow-features=test-dummy-unstable build")
+    p.cargo("-Zallow-features=test-dummy-unstable check")
         .masquerade_as_nightly_cargo(&["allow-features", "test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("-Zallow-features=test-dummy-unstable,print-im-a-teapot -Zprint-im-a-teapot build")
+    p.cargo("-Zallow-features=test-dummy-unstable,print-im-a-teapot -Zprint-im-a-teapot check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
             "test-dummy-unstable",
@@ -252,7 +252,7 @@ fn allow_features() {
         .with_stdout("im-a-teapot = true")
         .run();
 
-    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot build")
+    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
             "test-dummy-unstable",
@@ -266,7 +266,7 @@ error: the feature `print-im-a-teapot` is not in the list of allowed features: [
         )
         .run();
 
-    p.cargo("-Zallow-features= build")
+    p.cargo("-Zallow-features= check")
         .masquerade_as_nightly_cargo(&["allow-features", "test-dummy-unstable"])
         .with_status(101)
         .with_stderr(
@@ -295,22 +295,23 @@ fn allow_features_to_rustc() {
         .file(
             "src/lib.rs",
             r#"
+                #![allow(internal_features)]
                 #![feature(test_2018_feature)]
             "#,
         )
         .build();
 
-    p.cargo("-Zallow-features= build")
+    p.cargo("-Zallow-features= check")
         .masquerade_as_nightly_cargo(&["allow-features"])
         .with_status(101)
         .with_stderr_contains("[..]E0725[..]")
         .run();
 
-    p.cargo("-Zallow-features=test_2018_feature build")
+    p.cargo("-Zallow-features=test_2018_feature check")
         .masquerade_as_nightly_cargo(&["allow-features"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
@@ -342,7 +343,7 @@ fn allow_features_in_cfg() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build")
+    p.cargo("check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
             "test-dummy-unstable",
@@ -350,13 +351,13 @@ fn allow_features_in_cfg() {
         ])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("-Zprint-im-a-teapot build")
+    p.cargo("-Zprint-im-a-teapot check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
             "test-dummy-unstable",
@@ -366,7 +367,7 @@ fn allow_features_in_cfg() {
         .with_stderr("[FINISHED] [..]")
         .run();
 
-    p.cargo("-Zunstable-options build")
+    p.cargo("-Zunstable-options check")
         .masquerade_as_nightly_cargo(&["allow-features", "test-dummy-unstable", "print-im-a-teapot"])
         .with_status(101)
         .with_stderr(
@@ -377,7 +378,7 @@ error: the feature `unstable-options` is not in the list of allowed features: [p
         .run();
 
     // -Zallow-features overrides .cargo/config
-    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot build")
+    p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
             "test-dummy-unstable",
@@ -391,7 +392,7 @@ error: the feature `print-im-a-teapot` is not in the list of allowed features: [
         )
         .run();
 
-    p.cargo("-Zallow-features= build")
+    p.cargo("-Zallow-features= check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
             "test-dummy-unstable",
@@ -426,17 +427,17 @@ fn nightly_feature_requires_nightly() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -483,18 +484,18 @@ fn nightly_feature_requires_nightly_in_dep() {
         )
         .file("a/src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
-[COMPILING] b [..]
+[CHECKING] a [..]
+[CHECKING] b [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -537,17 +538,17 @@ fn cant_publish() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build")
+    p.cargo("check")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
         .run();
 
-    p.cargo("build")
+    p.cargo("check")
         .with_status(101)
         .with_stderr(
             "\
@@ -581,7 +582,7 @@ fn z_flags_rejected() {
         )
         .file("src/lib.rs", "")
         .build();
-    p.cargo("build -Zprint-im-a-teapot")
+    p.cargo("check -Zprint-im-a-teapot")
         .with_status(101)
         .with_stderr(
             "error: the `-Z` flag is only accepted on the nightly \
@@ -590,18 +591,18 @@ fn z_flags_rejected() {
         )
         .run();
 
-    p.cargo("build -Zarg")
+    p.cargo("check -Zarg")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_status(101)
         .with_stderr("error: unknown `-Z` flag specified: arg")
         .run();
 
-    p.cargo("build -Zprint-im-a-teapot")
+    p.cargo("check -Zprint-im-a-teapot")
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stdout("im-a-teapot = true\n")
         .with_stderr(
             "\
-[COMPILING] a [..]
+[CHECKING] a [..]
 [FINISHED] [..]
 ",
         )
@@ -610,7 +611,10 @@ fn z_flags_rejected() {
 
 #[cargo_test]
 fn publish_allowed() {
-    let registry = registry::init();
+    let registry = registry::RegistryBuilder::new()
+        .http_api()
+        .http_index()
+        .build();
 
     let p = project()
         .file(
@@ -627,16 +631,26 @@ fn publish_allowed() {
         .file("src/lib.rs", "")
         .build();
 
-    // HACK: Inject `a` directly into the index so `publish` won't block for it to be in
-    // the index.
-    //
-    // This is to ensure we can verify the Summary we post to the registry as doing so precludes
-    // the registry from processing the publish.
-    Package::new("a", "0.0.1").file("src/lib.rs", "").publish();
-
     p.cargo("publish")
         .replace_crates_io(registry.index_url())
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
+        .with_stderr(
+            "\
+[UPDATING] [..]
+[WARNING] [..]
+[..]
+[PACKAGING] a v0.0.1 [..]
+[VERIFYING] a v0.0.1 [..]
+[COMPILING] a v0.0.1 [..]
+[FINISHED] [..]
+[PACKAGED] [..]
+[UPLOADING] a v0.0.1 [..]
+[UPLOADED] a v0.0.1 to registry `crates-io`
+note: Waiting for `a v0.0.1` to be available at registry `crates-io`.
+You may press ctrl-c to skip waiting; the crate should be available shortly.
+[PUBLISHED] a v0.0.1 at registry `crates-io`
+",
+        )
         .run();
 }
 
