@@ -1,6 +1,6 @@
 //! Tests for credential-process.
 
-use cargo_test_support::registry::{Package, TestRegistry};
+use cargo_test_support::registry::TestRegistry;
 use cargo_test_support::{basic_manifest, cargo_process, paths, project, registry, Project};
 use std::fs::{self, read_to_string};
 
@@ -69,6 +69,8 @@ or use environment variable CARGO_REGISTRIES_ALTERNATIVE_TOKEN
 fn warn_both_token_and_process() {
     // Specifying both credential-process and a token in config should issue a warning.
     let _server = registry::RegistryBuilder::new()
+        .http_api()
+        .http_index()
         .alternative()
         .no_configure_token()
         .build();
@@ -77,7 +79,7 @@ fn warn_both_token_and_process() {
             ".cargo/config",
             r#"
                 [registries.alternative]
-                token = "sekrit"
+                token = "alternative-sekrit"
                 credential-process = "false"
             "#,
         )
@@ -96,16 +98,6 @@ fn warn_both_token_and_process() {
         .file("src/lib.rs", "")
         .build();
 
-    // HACK: Inject `foo` directly into the index so `publish` won't block for it to be in
-    // the index.
-    //
-    // This is to ensure we can verify the Summary we post to the registry as doing so precludes
-    // the registry from processing the publish.
-    Package::new("foo", "0.1.0")
-        .file("src/lib.rs", "")
-        .alternative(true)
-        .publish();
-
     p.cargo("publish --no-verify --registry alternative -Z credential-process")
         .masquerade_as_nightly_cargo(&["credential-process"])
         .with_status(101)
@@ -127,7 +119,7 @@ Only one of these values may be set, remove one or the other to proceed.
             credential-process = "false"
 
             [registries.alternative]
-            token = "sekrit"
+            token = "alternative-sekrit"
         "#,
     );
     p.cargo("publish --no-verify --registry alternative -Z credential-process")
@@ -138,7 +130,10 @@ Only one of these values may be set, remove one or the other to proceed.
 [PACKAGING] foo v0.1.0 [..]
 [PACKAGED] [..]
 [UPLOADING] foo v0.1.0 [..]
-[UPDATING] [..]
+[UPLOADED] foo v0.1.0 [..]
+note: Waiting [..]
+You may press ctrl-c [..]
+[PUBLISHED] foo v0.1.0 [..]
 ",
         )
         .run();
@@ -230,7 +225,10 @@ fn publish() {
 [PACKAGING] foo v0.1.0 [..]
 [PACKAGED] [..]
 [UPLOADING] foo v0.1.0 [..]
-[UPDATING] [..]
+[UPLOADED] foo v0.1.0 [..]
+note: Waiting [..]
+You may press ctrl-c [..]
+[PUBLISHED] foo v0.1.0 [..]
 ",
         )
         .run();
@@ -270,7 +268,7 @@ the credential-process configuration value must pass the \
 
     cargo_process("logout -Z credential-process")
         .replace_crates_io(registry.index_url())
-        .masquerade_as_nightly_cargo(&["credential-process", "cargo-logout"])
+        .masquerade_as_nightly_cargo(&["credential-process"])
         .with_status(101)
         .with_stderr(
             "\
@@ -379,12 +377,15 @@ fn logout() {
     .unwrap();
 
     cargo_process("logout -Z credential-process")
-        .masquerade_as_nightly_cargo(&["credential-process", "cargo-logout"])
+        .masquerade_as_nightly_cargo(&["credential-process"])
         .replace_crates_io(server.index_url())
         .with_stderr(
             "\
 token for `crates-io` has been erased!
 [LOGOUT] token for `crates-io` has been removed from local storage
+[NOTE] This does not revoke the token on the registry server.
+    If you need to revoke the token, visit <https://crates.io/me> \
+    and follow the instructions there.
 ",
         )
         .run();
