@@ -1,36 +1,68 @@
 //! Tests for profiles defined in config files.
 
+use cargo::util::toml::TomlDebugInfo;
 use cargo_test_support::paths::CargoPathExt;
 use cargo_test_support::registry::Package;
 use cargo_test_support::{basic_lib_manifest, paths, project};
 
+// TODO: this should be remove once -Zprofile-rustflags is stabilized
 #[cargo_test]
-fn named_profile_gated() {
-    // Named profile in config requires enabling in Cargo.toml.
+fn rustflags_works_with_zflag() {
     let p = project()
-        .file("src/lib.rs", "")
         .file(
-            ".cargo/config",
+            "Cargo.toml",
             r#"
-            [profile.foo]
-            inherits = 'dev'
-            opt-level = 1
+                [package]
+                name = "foo"
+                version = "0.0.1"
+            "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .file(
+            ".cargo/config.toml",
+            r#"
+                [profile.dev]
+                rustflags = ["-C", "link-dead-code=yes"]
             "#,
         )
         .build();
-    p.cargo("build --profile foo -Zunstable-options")
-        .masquerade_as_nightly_cargo()
+
+    p.cargo("check -v")
+        .masquerade_as_nightly_cargo(&["profile-rustflags"])
+        .with_status(101)
+        .with_stderr_contains("[..]feature `profile-rustflags` is required[..]")
+        .run();
+
+    p.cargo("check -v -Zprofile-rustflags")
+        .masquerade_as_nightly_cargo(&["profile-rustflags"])
         .with_stderr(
             "\
-[ERROR] config profile `foo` is not valid (defined in `[..]/foo/.cargo/config`)
+[CHECKING] foo [..]
+[RUNNING] `rustc --crate-name foo [..] -C link-dead-code=yes [..]
+[FINISHED] [..]
+",
+        )
+        .run();
 
-Caused by:
-  feature `named-profiles` is required
+    p.change_file(
+        ".cargo/config.toml",
+        r#"
+            [unstable]
+            profile-rustflags = true
 
-  consider adding `cargo-features = [\"named-profiles\"]` to the manifest
+            [profile.dev]
+            rustflags = ["-C", "link-dead-code=yes"]
+        "#,
+    );
+
+    p.cargo("check -v")
+        .masquerade_as_nightly_cargo(&["profile-rustflags"])
+        .with_stderr(
+            "\
+[FRESH] foo [..]
+[FINISHED] [..]
 ",
         )
-        .with_status(101)
         .run();
 }
 
@@ -198,7 +230,7 @@ fn profile_config_override_spec_multiple() {
         .with_stderr(
             "\
 [ERROR] multiple package overrides in profile `dev` match package `bar v0.5.0 ([..])`
-found package specs: bar, bar:0.5.0",
+found package specs: bar, bar@0.5.0",
         )
         .run();
 }
@@ -342,7 +374,7 @@ fn named_config_profile() {
     // foo -> middle -> bar -> dev
     // middle exists in Cargo.toml, the others in .cargo/config
     use super::config::ConfigBuilder;
-    use cargo::core::compiler::{CompileKind, CompileMode};
+    use cargo::core::compiler::CompileKind;
     use cargo::core::profiles::{Profiles, UnitFor};
     use cargo::core::{PackageId, Workspace};
     use cargo::util::interning::InternedString;
@@ -373,8 +405,6 @@ fn named_config_profile() {
     fs::write(
         paths::root().join("Cargo.toml"),
         r#"
-            cargo-features = ['named-profiles']
-
             [workspace]
 
             [profile.middle]
@@ -392,7 +422,7 @@ fn named_config_profile() {
         "#,
     )
     .unwrap();
-    let config = ConfigBuilder::new().nightly_features_allowed(true).build();
+    let config = ConfigBuilder::new().build();
     let profile_name = InternedString::new("foo");
     let ws = Workspace::new(&paths::root().join("Cargo.toml"), &config).unwrap();
     let profiles = Profiles::new(&ws, profile_name).unwrap();
@@ -402,31 +432,30 @@ fn named_config_profile() {
     let dep_pkg = PackageId::new("dep", "0.1.0", crates_io).unwrap();
 
     // normal package
-    let mode = CompileMode::Build;
     let kind = CompileKind::Host;
-    let p = profiles.get_profile(a_pkg, true, true, UnitFor::new_normal(), mode, kind);
+    let p = profiles.get_profile(a_pkg, true, true, UnitFor::new_normal(kind), kind);
     assert_eq!(p.name, "foo");
     assert_eq!(p.codegen_units, Some(2)); // "foo" from config
     assert_eq!(p.opt_level, "1"); // "middle" from manifest
-    assert_eq!(p.debuginfo, Some(1)); // "bar" from config
+    assert_eq!(p.debuginfo.to_option(), Some(TomlDebugInfo::Limited)); // "bar" from config
     assert_eq!(p.debug_assertions, true); // "dev" built-in (ignore build-override)
     assert_eq!(p.overflow_checks, true); // "dev" built-in (ignore package override)
 
     // build-override
-    let bo = profiles.get_profile(a_pkg, true, true, UnitFor::new_host(false), mode, kind);
+    let bo = profiles.get_profile(a_pkg, true, true, UnitFor::new_host(false, kind), kind);
     assert_eq!(bo.name, "foo");
     assert_eq!(bo.codegen_units, Some(6)); // "foo" build override from config
     assert_eq!(bo.opt_level, "0"); // default to zero
-    assert_eq!(bo.debuginfo, Some(1)); // SAME as normal
+    assert_eq!(bo.debuginfo.to_option(), Some(TomlDebugInfo::Limited)); // SAME as normal
     assert_eq!(bo.debug_assertions, false); // "foo" build override from manifest
     assert_eq!(bo.overflow_checks, true); // SAME as normal
 
     // package overrides
-    let po = profiles.get_profile(dep_pkg, false, true, UnitFor::new_normal(), mode, kind);
+    let po = profiles.get_profile(dep_pkg, false, true, UnitFor::new_normal(kind), kind);
     assert_eq!(po.name, "foo");
     assert_eq!(po.codegen_units, Some(7)); // "foo" package override from config
     assert_eq!(po.opt_level, "1"); // SAME as normal
-    assert_eq!(po.debuginfo, Some(1)); // SAME as normal
+    assert_eq!(po.debuginfo.to_option(), Some(TomlDebugInfo::Limited)); // SAME as normal
     assert_eq!(po.debug_assertions, true); // SAME as normal
     assert_eq!(po.overflow_checks, false); // "middle" package override from manifest
 }
@@ -438,7 +467,6 @@ fn named_env_profile() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
             [package]
             name = "foo"
             version = "0.1.0"
@@ -447,8 +475,7 @@ fn named_env_profile() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("build -v -Zunstable-options --profile=other")
-        .masquerade_as_nightly_cargo()
+    p.cargo("build -v --profile=other")
         .env("CARGO_PROFILE_OTHER_CODEGEN_UNITS", "1")
         .env("CARGO_PROFILE_OTHER_INHERITS", "dev")
         .with_stderr_contains("[..]-C codegen-units=1 [..]")
@@ -457,7 +484,8 @@ fn named_env_profile() {
 
 #[cargo_test]
 fn test_with_dev_profile() {
-    // `cargo test` uses "dev" profile for dependencies.
+    // The `test` profile inherits from `dev` for both local crates and
+    // dependencies.
     Package::new("somedep", "1.0.0").publish();
     let p = project()
         .file(
@@ -483,8 +511,9 @@ fn test_with_dev_profile() {
 [COMPILING] somedep v1.0.0
 [RUNNING] `rustc --crate-name somedep [..]-C debuginfo=0[..]
 [COMPILING] foo v0.1.0 [..]
-[RUNNING] `rustc --crate-name foo [..]-C debuginfo=2[..]
+[RUNNING] `rustc --crate-name foo [..]-C debuginfo=0[..]
 [FINISHED] [..]
+[EXECUTABLE] `[..]/target/debug/deps/foo-[..][EXE]`
 ",
         )
         .run();
