@@ -466,6 +466,80 @@ foo v0.1.0 ([..]/foo)
 └── build_target_dep v1.0.0
 [dev-dependencies]
 └── devdep v1.0.0
+",
+        )
+        .run();
+
+    // no-proc-macro
+    p.cargo("tree --target=all -e no-proc-macro")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+├── hostdep v1.0.0
+└── targetdep v1.0.0
+[build-dependencies]
+├── build_host_dep v1.0.0
+│   ├── hostdep v1.0.0
+│   └── targetdep v1.0.0
+└── build_target_dep v1.0.0
+[dev-dependencies]
+└── devdep v1.0.0
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn no_selected_target_dependency() {
+    // --target flag
+    if cross_compile::disabled() {
+        return;
+    }
+    Package::new("targetdep", "1.0.0").publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [target.'{alt}'.dependencies]
+                targetdep = "1.0"
+
+                "#,
+                alt = alternate(),
+            ),
+        )
+        .file("src/lib.rs", "")
+        .file("build.rs", "fn main() {}")
+        .build();
+
+    p.cargo("tree")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+",
+        )
+        .run();
+
+    p.cargo("tree -i targetdep")
+        .with_stderr(
+            "\
+[WARNING] nothing to print.
+
+To find dependencies that require specific target platforms, \
+try to use option `--target all` first, and then narrow your search scope accordingly.
+",
+        )
+        .run();
+    p.cargo("tree -i targetdep --target all")
+        .with_stdout(
+            "\
+targetdep v1.0.0
+└── foo v0.1.0 ([..]/foo)
 ",
         )
         .run();
@@ -476,6 +550,10 @@ fn dep_kinds() {
     Package::new("inner-devdep", "1.0.0").publish();
     Package::new("inner-builddep", "1.0.0").publish();
     Package::new("inner-normal", "1.0.0").publish();
+    Package::new("inner-pm", "1.0.0").proc_macro(true).publish();
+    Package::new("inner-buildpm", "1.0.0")
+        .proc_macro(true)
+        .publish();
     Package::new("normaldep", "1.0.0")
         .dep("inner-normal", "1.0")
         .dev_dep("inner-devdep", "1.0")
@@ -483,8 +561,10 @@ fn dep_kinds() {
         .publish();
     Package::new("devdep", "1.0.0")
         .dep("inner-normal", "1.0")
+        .dep("inner-pm", "1.0")
         .dev_dep("inner-devdep", "1.0")
         .build_dep("inner-builddep", "1.0")
+        .build_dep("inner-buildpm", "1.0")
         .publish();
     Package::new("builddep", "1.0.0")
         .dep("inner-normal", "1.0")
@@ -527,9 +607,11 @@ foo v0.1.0 ([..]/foo)
     └── inner-builddep v1.0.0
 [dev-dependencies]
 └── devdep v1.0.0
-    └── inner-normal v1.0.0
+    ├── inner-normal v1.0.0
+    └── inner-pm v1.0.0 (proc-macro)
     [build-dependencies]
-    └── inner-builddep v1.0.0
+    ├── inner-builddep v1.0.0
+    └── inner-buildpm v1.0.0 (proc-macro)
 ",
         )
         .run();
@@ -570,6 +652,23 @@ foo v0.1.0 ([..]/foo)
     [build-dependencies]
     └── inner-builddep v1.0.0
 [dev-dependencies]
+└── devdep v1.0.0
+    [build-dependencies]
+    ├── inner-builddep v1.0.0
+    └── inner-buildpm v1.0.0 (proc-macro)
+",
+        )
+        .run();
+
+    p.cargo("tree -e dev,build,no-proc-macro")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+[build-dependencies]
+└── builddep v1.0.0
+    [build-dependencies]
+    └── inner-builddep v1.0.0
+[dev-dependencies]
 └── devdep v1.0.0
     [build-dependencies]
     └── inner-builddep v1.0.0
@@ -941,6 +1040,104 @@ cat v2.0.0
         .run();
 }
 
+#[cargo_test]
+fn duplicates_with_target() {
+    // --target flag
+    if cross_compile::disabled() {
+        return;
+    }
+    Package::new("a", "1.0.0").publish();
+    Package::new("dog", "1.0.0").publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            a = "1.0"
+            dog = "1.0"
+
+            [build-dependencies]
+            a = "1.0"
+            dog = "1.0"
+
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file("build.rs", "fn main() {}")
+        .build();
+    p.cargo("tree -d").with_stdout("").run();
+
+    p.cargo("tree -d --target")
+        .arg(alternate())
+        .with_stdout("")
+        .run();
+
+    p.cargo("tree -d --target")
+        .arg(rustc_host())
+        .with_stdout("")
+        .run();
+
+    p.cargo("tree -d --target=all").with_stdout("").run();
+}
+
+#[cargo_test]
+fn duplicates_with_proc_macro() {
+    Package::new("dupe-dep", "1.0.0").publish();
+    Package::new("dupe-dep", "2.0.0").publish();
+    Package::new("proc", "1.0.0")
+        .proc_macro(true)
+        .dep("dupe-dep", "1.0")
+        .publish();
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            proc = "1.0"
+            dupe-dep = "2.0"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("tree")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+├── dupe-dep v2.0.0
+└── proc v1.0.0 (proc-macro)
+    └── dupe-dep v1.0.0
+",
+        )
+        .run();
+
+    p.cargo("tree --duplicates")
+        .with_stdout(
+            "\
+dupe-dep v1.0.0
+└── proc v1.0.0 (proc-macro)
+    └── foo v0.1.0 ([..]/foo)
+
+dupe-dep v2.0.0
+└── foo v0.1.0 ([..]/foo)
+",
+        )
+        .run();
+
+    p.cargo("tree --duplicates --edges no-proc-macro")
+        .with_stdout("")
+        .run();
+}
+
 #[cargo_test]
 fn charset() {
     let p = make_simple_proj();
@@ -966,6 +1163,23 @@ foo v0.1.0 ([..]/foo)
 #[cargo_test]
 fn format() {
     Package::new("dep", "1.0.0").publish();
+    Package::new("other-dep", "1.0.0").publish();
+
+    Package::new("dep_that_is_awesome", "1.0.0")
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "dep_that_is_awesome"
+                version = "1.0.0"
+
+                [lib]
+                name = "awesome_dep"
+            "#,
+        )
+        .file("src/lib.rs", "pub struct Straw;")
+        .publish();
+
     let p = project()
         .file(
             "Cargo.toml",
@@ -978,6 +1192,9 @@ fn format() {
 
             [dependencies]
             dep = {version="1.0", optional=true}
+            other-dep = {version="1.0", optional=true}
+            dep_that_is_awesome = {version="1.0", optional=true}
+
 
             [features]
             default = ["foo"]
@@ -985,7 +1202,7 @@ fn format() {
             bar = []
             "#,
         )
-        .file("src/lib.rs", "")
+        .file("src/main.rs", "")
         .build();
 
     p.cargo("tree --format <<<{p}>>>")
@@ -1022,8 +1239,21 @@ Caused by:
         .arg("{p} [{f}]")
         .with_stdout(
             "\
-foo v0.1.0 ([..]/foo) [bar,default,dep,foo]
-└── dep v1.0.0 []
+foo v0.1.0 ([..]/foo) [bar,default,dep,dep_that_is_awesome,foo,other-dep]
+├── dep v1.0.0 []
+├── dep_that_is_awesome v1.0.0 []
+└── other-dep v1.0.0 []
+",
+        )
+        .run();
+
+    p.cargo("tree")
+        .arg("--features=other-dep,dep_that_is_awesome")
+        .arg("--format={lib}")
+        .with_stdout(
+            "
+├── awesome_dep
+└── other_dep
 ",
         )
         .run();
@@ -1252,6 +1482,17 @@ foo v0.1.0 ([..]/foo)
 │   └── somedep v1.0.0
 │       └── optdep v1.0.0
 └── somedep v1.0.0 (*)
+",
+        )
+        .run();
+
+    // Old behavior + no-proc-macro
+    p.cargo("tree -e no-proc-macro")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+└── somedep v1.0.0
+    └── optdep v1.0.0
 ",
         )
         .run();
@@ -1262,6 +1503,16 @@ foo v0.1.0 ([..]/foo)
             "\
 somedep v1.0.0
 └── optdep v1.0.0
+",
+        )
+        .run();
+
+    // -p -e no-proc-macro
+    p.cargo("tree -p somedep -e no-proc-macro")
+        .with_stdout(
+            "\
+somedep v1.0.0
+└── optdep v1.0.0
 ",
         )
         .run();
@@ -1274,6 +1525,16 @@ somedep v1.0.0
 ├── foo v0.1.0 ([..]/foo)
 └── pm v1.0.0 (proc-macro)
     └── foo v0.1.0 ([..]/foo)
+",
+        )
+        .run();
+
+    // invert + no-proc-macro
+    p.cargo("tree -i somedep -e no-proc-macro")
+        .with_stdout(
+            "\
+somedep v1.0.0
+└── foo v0.1.0 ([..]/foo)
 ",
         )
         .run();
@@ -1290,6 +1551,15 @@ foo v0.1.0 ([..]/foo)
 │   └── somedep v1.0.0
 │       └── optdep v1.0.0
 └── somedep v1.0.0
+",
+        )
+        .run();
+
+    p.cargo("tree -e no-proc-macro")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+└── somedep v1.0.0
 ",
         )
         .run();
@@ -1314,6 +1584,15 @@ somedep v1.0.0
 somedep v1.0.0
 └── pm v1.0.0 (proc-macro)
     └── foo v0.1.0 ([..]/foo)
+",
+        )
+        .run();
+
+    p.cargo("tree -i somedep -e no-proc-macro")
+        .with_stdout(
+            "\
+somedep v1.0.0
+└── foo v0.1.0 ([..]/foo)
 ",
         )
         .run();
@@ -1397,8 +1676,8 @@ fn ambiguous_name() {
             "\
 error: There are multiple `dep` packages in your project, and the specification `dep` is ambiguous.
 Please re-run this command with `-p <spec>` where `<spec>` is one of the following:
-  dep:1.0.0
-  dep:2.0.0
+  dep@1.0.0
+  dep@2.0.0
 ",
         )
         .with_status(101)
@@ -1479,3 +1758,444 @@ b v0.1.0 ([..]/foo/b)
         )
         .run();
 }
+
+#[cargo_test]
+fn unknown_edge_kind() {
+    let p = project()
+        .file("Cargo.toml", "")
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("tree -e unknown")
+        .with_stderr(
+            "\
+[ERROR] unknown edge kind `unknown`, valid values are \
+\"normal\", \"build\", \"dev\", \
+\"no-normal\", \"no-build\", \"no-dev\", \"no-proc-macro\", \
+\"features\", or \"all\"
+",
+        )
+        .with_status(101)
+        .run();
+}
+
+#[cargo_test]
+fn mixed_no_edge_kinds() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("tree -e no-build,normal")
+        .with_stderr(
+            "\
+[ERROR] `normal` dependency kind cannot be mixed with \
+\"no-normal\", \"no-build\", or \"no-dev\" dependency kinds
+",
+        )
+        .with_status(101)
+        .run();
+
+    // `no-proc-macro` can be mixed with others
+    p.cargo("tree -e no-proc-macro,normal")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn depth_limit() {
+    let p = make_simple_proj();
+
+    p.cargo("tree --depth 0")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+[build-dependencies]
+[dev-dependencies]
+",
+        )
+        .run();
+
+    p.cargo("tree --depth 1")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+├── a v1.0.0
+└── c v1.0.0
+[build-dependencies]
+└── bdep v1.0.0
+[dev-dependencies]
+└── devdep v1.0.0
+",
+        )
+        .run();
+
+    p.cargo("tree --depth 2")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+├── a v1.0.0
+│   └── b v1.0.0
+└── c v1.0.0
+[build-dependencies]
+└── bdep v1.0.0
+    └── b v1.0.0 (*)
+[dev-dependencies]
+└── devdep v1.0.0
+    └── b v1.0.0 (*)
+",
+        )
+        .run();
+
+    // specify a package
+    p.cargo("tree -p bdep --depth 1")
+        .with_stdout(
+            "\
+bdep v1.0.0
+└── b v1.0.0
+",
+        )
+        .run();
+
+    // different prefix
+    p.cargo("tree --depth 1 --prefix depth")
+        .with_stdout(
+            "\
+0foo v0.1.0 ([..]/foo)
+1a v1.0.0
+1c v1.0.0
+1bdep v1.0.0
+1devdep v1.0.0
+",
+        )
+        .run();
+
+    // with edge-kinds
+    p.cargo("tree --depth 1 -e no-dev")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+├── a v1.0.0
+└── c v1.0.0
+[build-dependencies]
+└── bdep v1.0.0
+",
+        )
+        .run();
+
+    // invert
+    p.cargo("tree --depth 1 --invert c")
+        .with_stdout(
+            "\
+c v1.0.0
+├── b v1.0.0
+└── foo v0.1.0 ([..]/foo)
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn prune() {
+    let p = make_simple_proj();
+
+    p.cargo("tree --prune c")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+└── a v1.0.0
+    └── b v1.0.0
+[build-dependencies]
+└── bdep v1.0.0
+    └── b v1.0.0 (*)
+[dev-dependencies]
+└── devdep v1.0.0
+    └── b v1.0.0 (*)
+",
+        )
+        .run();
+
+    // multiple prune
+    p.cargo("tree --prune c --prune bdep")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+└── a v1.0.0
+    └── b v1.0.0
+[build-dependencies]
+[dev-dependencies]
+└── devdep v1.0.0
+    └── b v1.0.0 (*)
+",
+        )
+        .run();
+
+    // with edge-kinds
+    p.cargo("tree --prune c -e normal")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+└── a v1.0.0
+    └── b v1.0.0
+",
+        )
+        .run();
+
+    // pruning self does not works
+    p.cargo("tree --prune foo")
+        .with_stdout(
+            "\
+foo v0.1.0 ([..]/foo)
+├── a v1.0.0
+│   └── b v1.0.0
+│       └── c v1.0.0
+└── c v1.0.0
+[build-dependencies]
+└── bdep v1.0.0
+    └── b v1.0.0 (*)
+[dev-dependencies]
+└── devdep v1.0.0
+    └── b v1.0.0 (*)
+",
+        )
+        .run();
+
+    // dep not exist
+    p.cargo("tree --prune no-dep")
+        .with_stderr(
+            "\
+[ERROR] package ID specification `no-dep` did not match any packages
+
+<tab>Did you mean `bdep`?
+",
+        )
+        .with_status(101)
+        .run();
+}
+
+#[cargo_test]
+fn cyclic_features() {
+    // Check for stack overflow with cyclic features (oops!).
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "1.0.0"
+
+                [features]
+                a = ["b"]
+                b = ["a"]
+                default = ["a"]
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("tree -e features")
+        .with_stdout("foo v1.0.0 ([ROOT]/foo)")
+        .run();
+
+    p.cargo("tree -e features -i foo")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+├── foo feature \"a\"
+│   ├── foo feature \"b\"
+│   │   └── foo feature \"a\" (*)
+│   └── foo feature \"default\" (command-line)
+├── foo feature \"b\" (*)
+└── foo feature \"default\" (command-line)
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn dev_dep_cycle_with_feature() {
+    // Cycle with features and a dev-dependency.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "1.0.0"
+
+                [dev-dependencies]
+                bar = { path = "bar" }
+
+                [features]
+                a = ["bar/feat1"]
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "1.0.0"
+
+                [dependencies]
+                foo = { path = ".." }
+
+                [features]
+                feat1 = ["foo/a"]
+            "#,
+        )
+        .file("bar/src/lib.rs", "")
+        .build();
+
+    p.cargo("tree -e features --features a")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+[dev-dependencies]
+└── bar feature \"default\"
+    └── bar v1.0.0 ([ROOT]/foo/bar)
+        └── foo feature \"default\" (command-line)
+            └── foo v1.0.0 ([ROOT]/foo) (*)
+",
+        )
+        .run();
+
+    p.cargo("tree -e features --features a -i foo")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+├── foo feature \"a\" (command-line)
+│   └── bar feature \"feat1\"
+│       └── foo feature \"a\" (command-line) (*)
+└── foo feature \"default\" (command-line)
+    └── bar v1.0.0 ([ROOT]/foo/bar)
+        ├── bar feature \"default\"
+        │   [dev-dependencies]
+        │   └── foo v1.0.0 ([ROOT]/foo) (*)
+        └── bar feature \"feat1\" (*)
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn dev_dep_cycle_with_feature_nested() {
+    // Checks for an issue where a cyclic dev dependency tries to activate a
+    // feature on its parent that tries to activate the feature back on the
+    // dev-dependency.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "1.0.0"
+
+                [dev-dependencies]
+                bar = { path = "bar" }
+
+                [features]
+                a = ["bar/feat1"]
+                b = ["a"]
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "bar/Cargo.toml",
+            r#"
+                [package]
+                name = "bar"
+                version = "1.0.0"
+
+                [dependencies]
+                foo = { path = ".." }
+
+                [features]
+                feat1 = ["foo/b"]
+            "#,
+        )
+        .file("bar/src/lib.rs", "")
+        .build();
+
+    p.cargo("tree -e features")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+[dev-dependencies]
+└── bar feature \"default\"
+    └── bar v1.0.0 ([ROOT]/foo/bar)
+        └── foo feature \"default\" (command-line)
+            └── foo v1.0.0 ([ROOT]/foo) (*)
+",
+        )
+        .run();
+
+    p.cargo("tree -e features --features a -i foo")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+├── foo feature \"a\" (command-line)
+│   └── foo feature \"b\"
+│       └── bar feature \"feat1\"
+│           └── foo feature \"a\" (command-line) (*)
+├── foo feature \"b\" (*)
+└── foo feature \"default\" (command-line)
+    └── bar v1.0.0 ([ROOT]/foo/bar)
+        ├── bar feature \"default\"
+        │   [dev-dependencies]
+        │   └── foo v1.0.0 ([ROOT]/foo) (*)
+        └── bar feature \"feat1\" (*)
+",
+        )
+        .run();
+
+    p.cargo("tree -e features --features b -i foo")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+├── foo feature \"a\"
+│   └── foo feature \"b\" (command-line)
+│       └── bar feature \"feat1\"
+│           └── foo feature \"a\" (*)
+├── foo feature \"b\" (command-line) (*)
+└── foo feature \"default\" (command-line)
+    └── bar v1.0.0 ([ROOT]/foo/bar)
+        ├── bar feature \"default\"
+        │   [dev-dependencies]
+        │   └── foo v1.0.0 ([ROOT]/foo) (*)
+        └── bar feature \"feat1\" (*)
+",
+        )
+        .run();
+
+    p.cargo("tree -e features --features bar/feat1 -i foo")
+        .with_stdout(
+            "\
+foo v1.0.0 ([ROOT]/foo)
+├── foo feature \"a\"
+│   └── foo feature \"b\"
+│       └── bar feature \"feat1\" (command-line)
+│           └── foo feature \"a\" (*)
+├── foo feature \"b\" (*)
+└── foo feature \"default\" (command-line)
+    └── bar v1.0.0 ([ROOT]/foo/bar)
+        ├── bar feature \"default\"
+        │   [dev-dependencies]
+        │   └── foo v1.0.0 ([ROOT]/foo) (*)
+        └── bar feature \"feat1\" (command-line) (*)
+",
+        )
+        .run();
+}
